@@ -74,7 +74,7 @@ def print_node(node, detailed=False):
     # Node attributes
     print "  Node attributes:"
     for attribute in node.keys():
-        if attribute == "run_list" or attribute == "littlechef":
+        if attribute == "run_list" or attribute == "name":
             continue
         print "    {0}: {1}".format(attribute, node[attribute])
 
@@ -90,8 +90,11 @@ def get_recipes_in_cookbook(name):
     metadata_exists = False
     for cookbook_path in cookbook_paths:
         path = os.path.join(cookbook_path, name)
+        path_exists = os.path.exists(path)
         # cookbook exists if present in any of the cookbook paths
-        cookbook_exists = cookbook_exists or os.path.exists(path)
+        cookbook_exists = cookbook_exists or path_exists
+        if not path_exists:
+            continue
         # Now try to open metadata.json
         try:
             with open(os.path.join(path, 'metadata.json'), 'r') as f:
@@ -111,7 +114,7 @@ def get_recipes_in_cookbook(name):
                         'version': cookbook.get('version'),
                         'dependencies': cookbook.get('dependencies',
                                                         {}).keys(),
-                        'attributes': cookbook.get('attributes', {}).keys(),
+                        'attributes': cookbook.get('attributes', {}),
                         })
                 # When a recipe has no default recipe (libraries?),
                 # add one so that it is listed
@@ -122,7 +125,7 @@ def get_recipes_in_cookbook(name):
                         'version': cookbook.get('version'),
                         'dependencies': cookbook.get('dependencies',
                                                         {}).keys(),
-                        'attributes': cookbook.get('attributes', {}).keys()
+                        'attributes': cookbook.get('attributes', {})
                     })
             # Cookbook metadata.json was found, don't try next cookbook path
             # because metadata.json in site-cookbooks has preference
@@ -137,6 +140,14 @@ def get_recipes_in_cookbook(name):
     return recipes
 
 
+def get_recipes_in_role(rolename):
+    """Gets all recipes defined in a role's run_list"""
+    recipes = []
+    role = _get_role(rolename)
+    recipes.extend(get_recipes_in_node(role))
+    return recipes
+
+
 def get_recipes_in_node(node):
     """Gets the name of all recipes present in the run_list of a node"""
     recipes = []
@@ -168,6 +179,11 @@ def print_recipe(recipe):
     print "  attributes:   {0}".format(", ".join(recipe['attributes']))
 
 
+def get_roles_in_role(rolename):
+    """Gets all roles defined in a role's run_list"""
+    return get_roles_in_node(_get_role(rolename))
+
+
 def get_roles_in_node(node):
     """Gets the name of all roles found in the run_list of a node"""
     roles = []
@@ -203,7 +219,7 @@ def get_roles():
                 path = os.path.join(
                     root[len('roles/'):], filename[:-len('.json')])
                 roles.append(_get_role(path))
-    return roles
+    return sorted(roles, key=lambda x: x['fullname'])
 
 
 def print_role(role, detailed=True):
