@@ -12,7 +12,7 @@
 #See the License for the specific language governing permissions and
 #limitations under the License.
 #
-"""LittleChef: Configuration Management using Chef without a Chef Server"""
+"""LittleChef: Configuration Management using Chef Solo"""
 import ConfigParser
 import os
 import sys
@@ -74,18 +74,38 @@ def new_kitchen():
 
 
 @hosts('setup')
-def node(host):
-    """Select a node"""
-    if host == 'all':
+def node(*nodes):
+    """Selects and configures a list of nodes. 'all' configures all nodes"""
+    if not len(nodes) or nodes[0] == '':
+        abort('No node was given')
+    elif nodes[0] == 'all':
+        # Fetch all nodes and add them to env.hosts
         for node in lib.get_nodes():
             env.hosts.append(node['name'])
         if not len(env.hosts):
-            abort('No nodes found')
+            abort('No nodes found in /nodes/')
     else:
-        env.hosts = [host]
+        # A list of nodes was given
+        env.hosts = nodes
+    env.all_hosts = list(env.hosts)
+
+    # Check whether another command was given in addition to "node:"
+    execute = True
+    if 'node:' not in sys.argv[-1]:
+        execute = False
+    # If user didn't type recipe:X, role:Y or deploy_chef, just run configure
+    if execute:
+        for hostname in env.hosts:
+            env.host = hostname
+            env.host_string = hostname
+            lib.print_header("Configuring {0}".format(env.host))
+            # Read node data and configure node
+            node = lib.get_node(env.host)
+            chef.sync_node(node)
 
 
-def deploy_chef(gems="no", ask="yes", version="0.10"):
+def deploy_chef(gems="no", ask="yes", version="0.10",
+    distro_type=None, distro=None, stop_client='yes'):
     """Install chef-solo on a node"""
     if not env.host_string:
         abort('no node specified\nUsage: cook node:MYNODE deploy_chef')
@@ -93,7 +113,10 @@ def deploy_chef(gems="no", ask="yes", version="0.10"):
     if version not in chef_versions:
         abort('Wrong Chef version specified. Valid versions are {0}'.format(
             ", ".join(chef_versions)))
-    distro_type, distro = solo.check_distro()
+    if distro_type is None and distro is None:
+        distro_type, distro = solo.check_distro()
+    elif distro_type is None or distro is None:
+        abort('Must specify both or neither of distro_type and distro')
     if ask == "yes":
         message = '\nAre you sure you want to install Chef {0}'.format(version)
         message += ' at the node {0}'.format(env.host_string)
@@ -110,7 +133,7 @@ def deploy_chef(gems="no", ask="yes", version="0.10"):
             method = '{0} using "{1}" packages'.format(version, distro)
         print("Deploying Chef {0}...".format(method))
 
-    solo.install(distro_type, distro, gems, version)
+    solo.install(distro_type, distro, gems, version, stop_client)
     solo.configure()
 
 
@@ -123,7 +146,7 @@ def recipe(recipe):
     if not env.host_string:
         abort('no node specified\nUsage: cook node:MYNODE recipe:MYRECIPE')
     lib.print_header(
-        "Executing recipe '{0}' on node {1}".format(recipe, env.host_string))
+        "Applying recipe '{0}' on node {1}".format(recipe, env.host_string))
 
     # Now create configuration and sync node
     data = lib.get_node(env.host_string)
@@ -140,7 +163,7 @@ def role(role):
     if not env.host_string:
         abort('no node specified\nUsage: cook node:MYNODE role:MYROLE')
     lib.print_header(
-        "Applying role '{0}' to node {1}".format(role, env.host_string))
+        "Applying role '{0}' to {1}".format(role, env.host_string))
 
     # Now create configuration and sync node
     data = lib.get_node(env.host_string)
@@ -148,19 +171,31 @@ def role(role):
     chef.sync_node(data)
 
 
-def configure():
-    """Configure node using existing config file"""
-    # Check that a node has been selected
-    if not env.host_string:
-        msg = 'no node specified\n'
-        msg += 'Usage:\n  cook node:MYNODE configure'
-        msg += '\n  cook node:all configure'
-        abort(msg)
-    lib.print_header("Configuring {0}".format(env.host_string))
-
-    # Read node data and configure node
-    node = lib.get_node(env.host_string)
-    chef.sync_node(node)
+@hosts('setup')
+def get_ips():
+    """Ping all nodes and update their 'ipaddress' field"""
+    import subprocess
+    for node in lib.get_nodes():
+        # For each node, ping the hostname
+        env.host_string = node['name']
+        proc = subprocess.Popen(['ping', '-c', '1', node['name']],
+                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        resp, error = proc.communicate()
+        if not error:
+            # Get lines from output and parse the first line to get the IP
+            lines = resp.split("\n")
+            ip = lines[0].split()[2].lstrip("(").rstrip(")")
+            if not ip:
+                print "Warning: could not get IP address from node {0}".format(
+                    node['name'])
+                continue
+            print "Node {0} has IP {1}".format(node['name'], ip)
+            # Update with the ipaddress field in the corresponding node.json
+            del node['name']
+            node['ipaddress'] = ip
+            os.remove(chef.save_config(node, ip))
+        else:
+            print "Warning: could not resolve node {0}".format(node['name'])
 
 
 @hosts('api')
@@ -257,9 +292,9 @@ def _readconfig():
     missing_str = lambda m: ' and '.join(', '.join(m).rsplit(', ', 1))
     if not in_a_kitchen:
         msg = "Couldn't find {0}. ".format(missing_str(missing))
-        msg += "Are you are executing 'cook' outside of a kitchen?\n"\
+        msg += "Are you are executing 'fix' outside of a kitchen?\n"\
                "To create a new kitchen in the current directory "\
-               " type 'cook new_kitchen'"
+               " type 'fix new_kitchen'"
         abort(msg)
     config = ConfigParser.ConfigParser()
     config.read("auth.cfg")
@@ -313,7 +348,7 @@ def _readconfig():
         abort('You need to define a password or a keypair-file in auth.cfg.')
 
 
-# Only read config if cook is being used and we are not creating a new kitchen
+# Only read config if fix is being used and we are not creating a new kitchen
 import littlechef
 if littlechef.COOKING:
     # Called from command line
